package com.hz.pm.api.projectlib.manage;

import cn.hutool.core.lang.Assert;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.TypeReference;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.hz.pm.api.common.enumeration.CommonEnum;
import com.hz.pm.api.common.enumeration.ProjectProcessStageEnum;
import com.hz.pm.api.common.model.constant.BizConst;
import com.hz.pm.api.external.model.enumeration.MhUnitStripEnum;
import com.hz.pm.api.projectdeclared.manage.DefaultDeclaredProjectManage;
import com.hz.pm.api.projectdeclared.model.dto.DefaultDeclaredDTO;
import com.hz.pm.api.projectdeclared.model.dto.ProjectConditionDTO;
import com.hz.pm.api.projectdeclared.service.IProjectDraftService;
import com.hz.pm.api.projectlib.model.dto.ProjectDTO;
import com.hz.pm.api.projectlib.model.entity.MhProject;
import com.hz.pm.api.projectlib.model.entity.MhProjectSchemaTargetData;
import com.hz.pm.api.projectlib.model.entity.Project;
import com.hz.pm.api.projectlib.model.enumeration.ProjectStatusEnum;
import com.hz.pm.api.projectlib.model.req.DeclaredProjectListReq;
import com.hz.pm.api.projectlib.model.vo.DeclaredProjectListVO;
import com.hz.pm.api.projectlib.model.vo.MhProjectBaseInfoVO;
import com.hz.pm.api.projectlib.service.IMhProjectSchemaTargetDataService;
import com.hz.pm.api.projectlib.service.IMhProjectService;
import com.hz.pm.api.projectlib.service.IProjectService;
import com.hz.pm.api.staging.enums.MsgTypeEnum;
import com.hz.pm.api.sys.manage.NoticeManage;
import com.hz.pm.api.todocenter.constant.WorkNoticeConst;
import com.hz.pm.api.user.helper.MhUnitCache;
import com.hz.pm.api.user.model.dto.UnitDTO;
import com.hz.pm.api.user.security.model.UserInfoDetails;
import com.hz.pm.api.user.util.LoginUserUtil;
import com.ningdatech.basic.exception.BizException;
import com.ningdatech.basic.function.VUtils;
import com.ningdatech.basic.model.PageVo;
import com.ningdatech.basic.util.CollUtils;
import com.wflow.bean.entity.WflowModels;
import com.wflow.workflow.bean.dto.OrgInfoDTO;
import com.wflow.workflow.bean.vo.ProcessStartParamsVo;
import com.wflow.workflow.service.ProcessInstanceService;
import com.wflow.workflow.service.ProcessModelService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * <p>
 * MhProjectManage
 * </p>
 *
 * @author WendyYang
 * @since 19:06 2024/1/30
 */
@Slf4j
@Component
@RequiredArgsConstructor
public class DeclaredRecordManage {

    private final IMhProjectService mhProjectService;
    private final IMhProjectSchemaTargetDataService mhProjectSchemaTargetDataService;
    private final IProjectService projectService;
    private final MhUnitCache mhUnitCache;
    private final NoticeManage noticeManage;
    private final IProjectDraftService projectDraftService;
    private final ProcessModelService processModelService;
    private final DefaultDeclaredProjectManage defaultDeclaredProjectManage;
    private final ProcessInstanceService processInstanceService;
    private final ProjectLibManage projectLibManage;

    private LambdaQueryWrapper<MhProject> buildQuery(DeclaredProjectListReq req) {
        LambdaQueryWrapper<MhProject> query = Wrappers.lambdaQuery(MhProject.class)
                .like(StrUtil.isNotBlank(req.getProjectName()), MhProject::getProjectName, req.getProjectName())
                .like(StrUtil.isNotBlank(req.getDeclaredUnit()), MhProject::getUnitName, req.getDeclaredUnit());
        if (req.getCreateDateMin() != null) {
            query.ge(MhProject::getCreateTime, req.getCreateDateMin());
        }
        if (req.getCreateDateMax() != null) {
            query.lt(MhProject::getCreateTime, req.getCreateDateMax().minusDays(1));
        }
        UserInfoDetails user = LoginUserUtil.loginUserDetail();
        if (!user.getSuperAdmin() && !user.getRegionAdmin()) {
            query.eq(MhProject::getUnitId, user.getMhUnitId());
        }
        if (req.getUnitStrip() != null) {
            query.eq(MhProject::getUnitStrip, req.getUnitStrip());
        }
        return query;
    }

    public PageVo<DeclaredProjectListVO> pageDeclaredProject(DeclaredProjectListReq req) {
        LambdaQueryWrapper<MhProject> query = buildQuery(req);
        Page<MhProject> page = mhProjectService.page(req.page(), query);
        if (page.getTotal() == 0) {
            return PageVo.empty();
        }
        List<MhProject> records = page.getRecords();
        List<String> projectCodes = CollUtils.fieldList(records, MhProject::getProjectCode);
        Map<String, Project> projectMap;
        if (!projectCodes.isEmpty()) {
            List<Project> projects = projectService.listNewestByProjectCodes(projectCodes);
            projectMap = CollUtils.listToMap(projects, Project::getProjectCode);
        } else {
            projectMap = Collections.emptyMap();
        }
        List<DeclaredProjectListVO> data = records.stream().map(w -> {
            DeclaredProjectListVO item = new DeclaredProjectListVO();
            item.setMhProjectId(w.getId());
            Project project = projectMap.get(w.getProjectCode());
            if (project == null) {
                item.setStatus(null);
                item.setStage(null);
            } else {
                item.setStatus(project.getStatus());
                item.setStage(project.getStage());
            }
            item.setUnitStrip(w.getUnitStrip());
            item.setUnitStripName(MhUnitStripEnum.getVal(w.getUnitStrip()));
            item.setDeclaredUnit(w.getUnitName());
            item.setProjectCode(w.getProjectCode());
            item.setProjectName(w.getProjectName());
            item.setDeclaredAmount(w.getTotalMoney());
            item.setCreateTime(w.getCreateTime());
            return item;
        }).collect(Collectors.toList());
        return PageVo.of(data, page.getTotal());
    }

    public MhProjectBaseInfoVO getProjectBaseInfo(Long mhProjectId) {
        MhProject mhProject = mhProjectService.getById(mhProjectId);
        if (mhProject == null) {
            throw BizException.wrap("项目不存在");
        }
        MhProjectBaseInfoVO detail = new MhProjectBaseInfoVO();
        detail.setMhProjectId(mhProject.getId());
        detail.setProjectName(mhProject.getProjectName());
        detail.setDeclaredUnit(mhProject.getUnitName());
        detail.setDeclaredUnitId(mhProject.getUnitId());
        detail.setTotalInvestAmount(mhProject.getTotalMoney());
        detail.setReviewInvestAmount(mhProject.getMoney());
        MhProjectSchemaTargetData schemaTargetData = mhProjectSchemaTargetDataService.getByMhProjectId(mhProjectId);
        if (schemaTargetData != null) {
            detail.setTotalInvestBudgetAmount(schemaTargetData.getMoney());
        }
        UnitDTO unit = mhUnitCache.getById(mhProject.getId());
        if (unit != null) {
            detail.setDeclaredUnitUnifiedSocialCreditCode(unit.getUnifiedSocialCreditCode());
        }
        return detail;
    }

    @Transactional(rollbackFor = Exception.class)
    public void startDeclaredRecord(DefaultDeclaredDTO req) {
        UserInfoDetails user = LoginUserUtil.loginUserDetail();
        ProjectDTO projectInfo = req.getProjectInfo();

        projectInfo.setBuildOrgCode(user.getMhUnitIdStr());
        projectInfo.setBuildOrgName(user.getMhUnitName());

        // 项目名称去重
        defaultDeclaredProjectManage.checkDuplication(projectInfo);
        // 判断申报金额 是否等于总的 判断年度支付金额 是否等于总金额
        defaultDeclaredProjectManage.checkAmount(projectInfo);

        // 写死 是否有主管单位
        projectInfo.setIsHigherSuperOrg(CommonEnum.YES.getCode());
        projectInfo.setIsSuperOrg(CommonEnum.YES.getCode());

        //如果主管单位没有 那么主管单位就是自己
        if (CommonEnum.NO.getCode().equals(projectInfo.getIsSuperOrg())) {
            projectInfo.setSuperOrgCode(user.getMhUnitIdStr());
            projectInfo.setSuperOrg(user.getMhUnitName());
        }

        //如果是重新提交的话 判断下 项目是否存在
        if (Objects.nonNull(projectInfo.getId())) {
            //新申报的项目不允许带项目id
            projectInfo.setId(null);
        }

        WflowModels model = processModelService.getOne(Wrappers.lambdaQuery(WflowModels.class)
                .eq(WflowModels::getProcessType, ProjectProcessStageEnum.PROJECT_RECORD_APPROVAL_PROCESS.getCode())
                .last(BizConst.LIMIT_1));

        if (Objects.isNull(model)) {
            throw BizException.wrap("找不到立项备案审批流程配置");
        }

        ProcessStartParamsVo params = new ProcessStartParamsVo();
        params.setUser(defaultDeclaredProjectManage.buildUser(user.getUserId()));
        params.setProcessUsers(Collections.emptyMap());
        //放入条件判断的项目字段
        ProjectConditionDTO conditionDto = new ProjectConditionDTO();
        BeanUtils.copyProperties(req.getProjectInfo(), conditionDto);
        req.getFormData().putAll(
                JSON.parseObject(JSON.toJSONString(conditionDto), new TypeReference<Map<String, Object>>() {
                })
        );
        params.setFormData(req.getFormData());
        //开始申报
        // 获取发起单位、发起单位主管单位、发起单位上级主管条线单位信息
        Project project = new Project();
        BeanUtils.copyProperties(projectInfo, project);
        Map<String, OrgInfoDTO> orgModelMap = defaultDeclaredProjectManage.buildOrgModelMap(user.getUserId(), project);
        String instanceId = processInstanceService.startProcessLs(model, params, orgModelMap);
        log.info("申报项目成功 【{}】", instanceId);

        //如果是重新提交的话 判断下 项目是否存在
        //保存项目相关
        Project buildProject = projectLibManage.saveProjectInDeclared(projectInfo, instanceId, user.getUserId(),
                ProjectStatusEnum.NOT_APPROVED, ProjectStatusEnum.DECLARED_APPROVED_RECORD_AUDITING,
                ProjectProcessStageEnum.PROJECT_RECORD_APPROVAL_PROCESS);
        if (Objects.nonNull(projectInfo.getDraftId())) {
            //如果是草稿箱提交 删除对应的草稿箱
            projectDraftService.removeById(projectInfo.getDraftId());
        }

        //发送给第一个审批人消息
        noticeManage.sendFirstUser(buildProject, model.getFormName(), instanceId,
                WorkNoticeConst.PASS_MSG_TEMPLATE, MsgTypeEnum.PROJECT_REVIEW);
    }

    @Transactional(rollbackFor = Exception.class)
    public String restartDeclaredRecord(DefaultDeclaredDTO req) {
        UserInfoDetails user = LoginUserUtil.loginUserDetail();
        ProjectDTO reqProj = req.getProjectInfo();
        Assert.notNull(reqProj.getId(), "提交失败，缺少项目ID!");
        reqProj.setBuildOrgCode(user.getMhUnitIdStr());
        reqProj.setBuildOrgName(user.getMhUnitName());

        Project oldProj = projectService.getById(reqProj.getId());
        Assert.notNull(oldProj, "提交失败，此项目不存在!");

        //项目名称去重
        if (StrUtil.isNotBlank(reqProj.getProjectName()) &&
                !reqProj.getProjectName().equals(oldProj.getProjectName())) {
            reqProj.setProjectCode(oldProj.getProjectCode());
            defaultDeclaredProjectManage.checkDuplication(reqProj);
        }

        // 写死 是否有主管单位
        oldProj.setIsHigherSuperOrg(CommonEnum.YES.getCode());
        oldProj.setIsSuperOrg(CommonEnum.YES.getCode());

        //如果主管单位没有 那么主管单位就是自己
        if (CommonEnum.NO.getCode().equals(reqProj.getIsSuperOrg())) {
            oldProj.setSuperOrgCode(user.getMhUnitIdStr());
            oldProj.setSuperOrg(user.getMhUnitName());
            reqProj.setSuperOrgCode(user.getMhUnitIdStr());
            reqProj.setSuperOrg(user.getMhUnitName());
        }

        //判断申报金额 是否等于总的 判断年度支付金额 是否等于总金额
        defaultDeclaredProjectManage.checkAmount(reqProj);
        ProjectProcessStageEnum processStageEnum = ProjectProcessStageEnum.PROJECT_RECORD_APPROVAL_PROCESS;
        WflowModels model = processModelService.getOne(Wrappers.lambdaQuery(WflowModels.class)
                .eq(WflowModels::getProcessType, processStageEnum.getCode())
                .last(BizConst.LIMIT_1));

        if (Objects.isNull(model)) {
            throw BizException.wrap("%s不存在", processStageEnum.getDesc());
        }

        //首先要判断 项目当前状态 是不是 单位内部拒绝
        VUtils.isTrue(!ProjectStatusEnum.DECLARED_APPROVED_RECORD_FAILED.eq(oldProj.getStatus()) ||
                        !ProjectStatusEnum.NOT_APPROVED.eq(oldProj.getStage()))
                .throwMessage("提交失败，该项目不是立项备案审核不通过状态或者未立项阶段");

        ProcessStartParamsVo params = new ProcessStartParamsVo();
        params.setUser(defaultDeclaredProjectManage.buildUser(user.getUserId()));
        params.setProcessUsers(Collections.emptyMap());

        //把条件值给放入工作流
        defaultDeclaredProjectManage.buildCondition(params, req);

        // 获取发起单位、发起单位主管单位、发起单位上级主管条线单位信息
        Project project = new Project();
        BeanUtils.copyProperties(reqProj, project);
        Map<String, OrgInfoDTO> orgModelMap = defaultDeclaredProjectManage.buildOrgModelMap(user.getUserId(), project);
        String instId = processInstanceService.startProcessLs(model, params, orgModelMap);
        log.info("立项备案重新提交 【{}】", instId);

        //保存项目相关
        Project newProj = projectLibManage.saveProjectInDeclared(reqProj, instId, user.getUserId(),
                ProjectStatusEnum.NOT_APPROVED,
                ProjectStatusEnum.DECLARED_APPROVED_RECORD_AUDITING,
                processStageEnum);

        //发送给第一个审批人消息
        noticeManage.sendFirstUser(newProj, model.getFormName(), instId,
                WorkNoticeConst.PASS_MSG_TEMPLATE, MsgTypeEnum.PROJECT_REVIEW);

        return instId;
    }


}
